@@ -3,11 +3,15 @@
 from typing import Optional
 
 import torch
+from linear_operator.operators import (
+    DiagLinearOperator,
+    InterpolatedLinearOperator,
+    PsdSumLinearOperator,
+    RootLinearOperator,
+)
 
 from ..constraints import Interval, Positive
-from ..lazy import DiagLazyTensor, InterpolatedLazyTensor, PsdSumLazyTensor, RootLazyTensor
 from ..priors import Prior
-from ..utils.broadcasting import _mul_broadcast_shape
 from .kernel import Kernel
 
 
@@ -90,17 +94,17 @@ class IndexKernel(Kernel):
     @property
     def covar_matrix(self):
         var = self.var
-        res = PsdSumLazyTensor(RootLazyTensor(self.covar_factor), DiagLazyTensor(var))
+        res = PsdSumLinearOperator(RootLinearOperator(self.covar_factor), DiagLinearOperator(var))
         return res
 
     def forward(self, i1, i2, **params):
 
         i1, i2 = i1.long(), i2.long()
         covar_matrix = self._eval_covar_matrix()
-        batch_shape = _mul_broadcast_shape(i1.shape[:-2], i2.shape[:-2], self.batch_shape)
+        batch_shape = torch.broadcast_shapes(i1.shape[:-2], i2.shape[:-2], self.batch_shape)
 
-        res = InterpolatedLazyTensor(
-            base_lazy_tensor=covar_matrix,
+        res = InterpolatedLinearOperator(
+            base_linear_op=covar_matrix,
             left_interp_indices=i1.expand(batch_shape + i1.shape[-2:]),
             right_interp_indices=i2.expand(batch_shape + i2.shape[-2:]),
         )
