@@ -63,13 +63,13 @@ class PiecewisePolynomialKernel(Kernel):
         >>> covar_module = gpytorch.kernels.ScaleKernel(
                             gpytorch.kernels.PiecewisePolynomialKernel(q = 2, ard_num_dims=5)
                             )
-        >>> covar = covar_module(x)  # Output: LazyTensor of size (10 x 10)
+        >>> covar = covar_module(x)  # Output: LinearOperator of size (10 x 10)
         >>> batch_x = torch.randn(2, 10, 5)
         >>> # Batch: different lengthscale for each batch
         >>> covar_module = gpytorch.kernels.ScaleKernel(
             gpytorch.kernels.PiecewisePolynomialKernel(q = 2, batch_shape=torch.Size([2]))
             )
-        >>> covar = covar_module(batch_x)  # Output: LazyTensor of size (2 x 10 x 10)
+        >>> covar = covar_module(batch_x)  # Output: LinearOperator of size (2 x 10 x 10)
     """
     has_lengthscale = True
 
