@@ -1,16 +1,15 @@
 #!/usr/bin/env python3
 
 import torch
+from linear_operator import LinearOperator, to_linear_operator
+from linear_operator.utils.getitem import _noop_index
 
 from .. import beta_features, settings
-from ..utils import broadcasting
-from ..utils.getitem import _noop_index
+from ..utils import deprecation
 from ..utils.memoize import cached
-from .lazy_tensor import LazyTensor
-from .non_lazy_tensor import lazify
 
 
-class LazyEvaluatedKernelTensor(LazyTensor):
+class LazyEvaluatedKernelTensor(LinearOperator):
     _check_size = False
 
     def _check_args(self, x1, x2, kernel, last_dim_is_batch=False, **params):
@@ -43,10 +42,75 @@ class LazyEvaluatedKernelTensor(LazyTensor):
 
     def _set_requires_grad(self, val):
         super()._set_requires_grad(val)
-        # The behavior that differs from the base LazyTensor setter
+        # The behavior that differs from the base LinearOperator setter
         for param in self.kernel.parameters():
             param.requires_grad_(val)
 
+    def _bilinear_derivative(self, left_vecs, right_vecs):
+        # This _bilinear_derivative computes the kernel in chunks
+        # It is only used when we are using kernel checkpointing
+        # It won't be called if checkpointing is off
+        split_size = beta_features.checkpoint_kernel.value()
+        if not split_size:
+            raise RuntimeError(
+                "Should not have ended up in LazyEvaluatedKernelTensor._bilinear_derivative without kernel "
+                "checkpointing. This is probably a bug in GPyTorch."
+            )
+
+        x1 = self.x1.detach().requires_grad_(True)
+        x2 = self.x2.detach().requires_grad_(True)
+
+        # Break objects into chunks
+        sub_x1s = [sub_x1.detach() for sub_x1 in torch.split(x1, split_size, dim=-2)]
+        sub_left_vecss = torch.split(left_vecs, split_size, dim=-2)
+        # Compute the gradient in chunks
+        for sub_x1, sub_left_vecs in zip(sub_x1s, sub_left_vecss):
+            sub_x1.requires_grad_(True)
+            with torch.enable_grad(), settings.lazily_evaluate_kernels(False):
+                sub_kernel_matrix = to_linear_operator(
+                    self.kernel(
+                        sub_x1,
+                        x2,
+                        diag=False,
+                        last_dim_is_batch=self.last_dim_is_batch,
+                        **self.params,
+                    )
+                )
+            sub_grad_outputs = tuple(sub_kernel_matrix._bilinear_derivative(sub_left_vecs, right_vecs))
+            sub_kernel_outputs = tuple(sub_kernel_matrix.representation())
+            torch.autograd.backward(sub_kernel_outputs, sub_grad_outputs)
+
+        x1.grad = torch.cat([sub_x1.grad.data for sub_x1 in sub_x1s], dim=-2)
+        return x1.grad, x2.grad
+
+    @cached(name="kernel_diag")
+    def _diagonal(self) -> torch.Tensor:
+        # Getting the diagonal of a kernel can be handled more efficiently by
+        # transposing the batch and data dimension before calling the kernel.
+        # Implementing it this way allows us to compute predictions more efficiently
+        # in cases where only the variances are required.
+        from ..kernels import Kernel
+
+        x1 = self.x1
+        x2 = self.x2
+
+        res = super(Kernel, self.kernel).__call__(
+            x1, x2, diag=True, last_dim_is_batch=self.last_dim_is_batch, **self.params
+        )
+
+        # Now we'll make sure that the shape we're getting from diag makes sense
+        if settings.debug.on():
+            expected_shape = self.shape[:-1]
+            if res.shape != expected_shape:
+                raise RuntimeError(
+                    "The kernel {} is not equipped to handle and diag. Expected size {}. "
+                    "Got size {}".format(self.kernel.__class__.__name__, expected_shape, res.shape)
+                )
+
+        if isinstance(res, LinearOperator):
+            res = res.to_dense()
+        return res.view(self.shape[:-1]).contiguous()
+
     def _expand_batch(self, batch_shape):
         return self.evaluate_kernel()._expand_batch(batch_shape)
 
@@ -170,7 +234,7 @@ class LazyEvaluatedKernelTensor(LazyTensor):
             sub_x1s = torch.split(x1, split_size, dim=-2)
             res = []
             for sub_x1 in sub_x1s:
-                sub_kernel_matrix = lazify(
+                sub_kernel_matrix = to_linear_operator(
                     self.kernel(
                         sub_x1,
                         x2,
@@ -184,43 +248,6 @@ class LazyEvaluatedKernelTensor(LazyTensor):
             res = torch.cat(res, dim=-2)
             return res
 
-    def _quad_form_derivative(self, left_vecs, right_vecs):
-        # This _quad_form_derivative computes the kernel in chunks
-        # It is only used when we are using kernel checkpointing
-        # It won't be called if checkpointing is off
-        split_size = beta_features.checkpoint_kernel.value()
-        if not split_size:
-            raise RuntimeError(
-                "Should not have ended up in LazyEvaluatedKernelTensor._quad_form_derivative without kernel "
-                "checkpointing. This is probably a bug in GPyTorch."
-            )
-
-        x1 = self.x1.detach().requires_grad_(True)
-        x2 = self.x2.detach().requires_grad_(True)
-
-        # Break objects into chunks
-        sub_x1s = [sub_x1.detach() for sub_x1 in torch.split(x1, split_size, dim=-2)]
-        sub_left_vecss = torch.split(left_vecs, split_size, dim=-2)
-        # Compute the gradient in chunks
-        for sub_x1, sub_left_vecs in zip(sub_x1s, sub_left_vecss):
-            sub_x1.requires_grad_(True)
-            with torch.enable_grad(), settings.lazily_evaluate_kernels(False):
-                sub_kernel_matrix = lazify(
-                    self.kernel(
-                        sub_x1,
-                        x2,
-                        diag=False,
-                        last_dim_is_batch=self.last_dim_is_batch,
-                        **self.params,
-                    )
-                )
-            sub_grad_outputs = tuple(sub_kernel_matrix._quad_form_derivative(sub_left_vecs, right_vecs))
-            sub_kernel_outputs = tuple(sub_kernel_matrix.representation())
-            torch.autograd.backward(sub_kernel_outputs, sub_grad_outputs)
-
-        x1.grad = torch.cat([sub_x1.grad.data for sub_x1 in sub_x1s], dim=-2)
-        return x1.grad, x2.grad
-
     @cached(name="size")
     def _size(self):
         if settings.debug.on():
@@ -245,23 +272,19 @@ class LazyEvaluatedKernelTensor(LazyTensor):
 
         # When we're using broadcasting
         else:
-            expected_size = broadcasting._matmul_broadcast_shape(
-                torch.Size([*x1.shape[:-2], num_rows, x1.size(-1)]),
-                torch.Size([*x2.shape[:-2], x2.size(-1), num_cols]),
-                error_msg="x1 and x2 were not broadcastable to a proper kernel shape. "
-                "Got x1.shape = {} and x2.shape = {}".format(str(x1.shape), str(x2.shape)),
-            )
-            expected_size = (
-                broadcasting._mul_broadcast_shape(
-                    expected_size[:-2],
-                    self.kernel.batch_shape,
-                    error_msg=(
-                        f"x1 and x2 were not broadcastable with kernel of batch_shape {self.kernel.batch_shape}. "
-                        f"Got x1.shape = {x1.shape} and x2.shape = {x2.shape}"
-                    ),
+            try:
+                if x1.size(-1) != x2.size(-1):
+                    raise RuntimeError
+
+                expected_size = torch.broadcast_shapes(
+                    x1.shape[:-2], x2.shape[:-2], self.kernel.batch_shape
+                ) + torch.Size([num_rows, num_cols])
+
+            except RuntimeError:
+                raise RuntimeError(
+                    f"x1 and x2 were not broadcastable with kernel of batch_shape {self.kernel.batch_shape}. "
+                    f"Got x1.shape = {x1.shape} and x2.shape = {x2.shape}"
                 )
-                + expected_size[-2:]
-            )
 
         # Handle when the last dim is batch
         if self.last_dim_is_batch:
@@ -291,41 +314,11 @@ class LazyEvaluatedKernelTensor(LazyTensor):
             **self.params,
         )
 
-    @cached(name="kernel_diag")
-    def diag(self):
-        """
-        Getting the diagonal of a kernel can be handled more efficiently by
-        transposing the batch and data dimension before calling the kernel.
-        Implementing it this way allows us to compute predictions more efficiently
-        in cases where only the variances are required.
-        """
-        from ..kernels import Kernel
-
-        x1 = self.x1
-        x2 = self.x2
-
-        res = super(Kernel, self.kernel).__call__(
-            x1, x2, diag=True, last_dim_is_batch=self.last_dim_is_batch, **self.params
-        )
-
-        # Now we'll make sure that the shape we're getting from diag makes sense
-        if settings.debug.on():
-            expected_shape = self.shape[:-1]
-            if res.shape != expected_shape:
-                raise RuntimeError(
-                    "The kernel {} is not equipped to handle and diag. Expected size {}. "
-                    "Got size {}".format(self.kernel.__class__.__name__, expected_shape, res.shape)
-                )
-
-        if isinstance(res, LazyTensor):
-            res = res.evaluate()
-        return res.view(self.shape[:-1]).contiguous()
-
     @cached(name="kernel_eval")
     def evaluate_kernel(self):
         """
-        NB: This is a meta LazyTensor, in the sense that evaluate can return
-        a LazyTensor if the kernel being evaluated does so.
+        NB: This is a meta LinearOperator, in the sense that evaluate can return
+        a LinearOperator if the kernel being evaluated does so.
         """
         x1 = self.x1
         x2 = self.x2
@@ -350,11 +343,7 @@ class LazyEvaluatedKernelTensor(LazyTensor):
                     "This is likely a bug in GPyTorch."
                 )
 
-        return lazify(res)
-
-    @cached
-    def evaluate(self):
-        return self.evaluate_kernel().evaluate()
+        return to_linear_operator(res)
 
     def repeat(self, *repeats):
         if len(repeats) == 1 and hasattr(repeats[0], "__iter__"):
@@ -375,7 +364,7 @@ class LazyEvaluatedKernelTensor(LazyTensor):
         # If we're checkpointing the kernel, we'll use chunked _matmuls defined in LazyEvaluatedKernelTensor
         if beta_features.checkpoint_kernel.value():
             return super().representation()
-        # Otherwise, we'll evaluate the kernel (or at least its LazyTensor representation) and use its
+        # Otherwise, we'll evaluate the kernel (or at least its LinearOperator representation) and use its
         # representation
         else:
             return self.evaluate_kernel().representation()
@@ -384,15 +373,19 @@ class LazyEvaluatedKernelTensor(LazyTensor):
         # If we're checkpointing the kernel, we'll use chunked _matmuls defined in LazyEvaluatedKernelTensor
         if beta_features.checkpoint_kernel.value():
             return super().representation_tree()
-        # Otherwise, we'll evaluate the kernel (or at least its LazyTensor representation) and use its
+        # Otherwise, we'll evaluate the kernel (or at least its LinearOperator representation) and use its
         # representation
         else:
             return self.evaluate_kernel().representation_tree()
 
+    @cached
+    def to_dense(self):
+        return self.evaluate_kernel().to_dense()
+
     def __getitem__(self, index):
         """
-        Supports subindexing of the matrix this LazyTensor represents. This may return either another
-        :obj:`gpytorch.lazy.LazyTensor` or a :obj:`torch.tensor` depending on the exact implementation.
+        Supports subindexing of the matrix this LinearOperator represents. This may return either another
+        :obj:`~linear_operator.operators.LinearOperator` or a :obj:`torch.tensor` depending on the exact implementation.
         """
         # Process the index
         index = index if isinstance(index, tuple) else (index,)
@@ -403,3 +396,12 @@ class LazyEvaluatedKernelTensor(LazyTensor):
             return self._getitem(row_index, col_index, *batch_indices)
         else:
             return super().__getitem__(index)
+
+
+deprecation._deprecated_renamed_method(
+    LazyEvaluatedKernelTensor, old_method_name="_quad_form_derivative", new_method_name="_bilinear_derivative"
+)
+deprecation._deprecated_renamed_method(LazyEvaluatedKernelTensor, old_method_name="diag", new_method_name="diagonal")
+deprecation._deprecated_renamed_method(
+    LazyEvaluatedKernelTensor, old_method_name="evaluate", new_method_name="to_dense"
+)
