@@ -6,10 +6,10 @@ from copy import deepcopy
 from typing import Any, Optional
 
 import torch
+from linear_operator.operators import ZeroLinearOperator
 from torch import Tensor
 
 from ..distributions import MultivariateNormal, base_distributions
-from ..lazy import ZeroLazyTensor
 from ..utils.warnings import GPInputWarning
 from .likelihood import Likelihood
 from .noise_models import FixedGaussianNoise, HomoskedasticNoise, Noise
@@ -38,7 +38,7 @@ class _GaussianLikelihoodBase(Likelihood):
         mean, variance = input.mean, input.variance
         num_event_dim = len(input.event_shape)
 
-        noise = self._shaped_noise_covar(mean.shape, *params, **kwargs).diag()
+        noise = self._shaped_noise_covar(mean.shape, *params, **kwargs).diagonal(dim1=-1, dim2=-2)
         # Potentially reshape the noise to deal with the multitask case
         noise = noise.view(*noise.shape[:-1], *input.event_shape)
 
@@ -49,7 +49,7 @@ class _GaussianLikelihoodBase(Likelihood):
         return res
 
     def forward(self, function_samples: Tensor, *params: Any, **kwargs: Any) -> base_distributions.Normal:
-        noise = self._shaped_noise_covar(function_samples.shape, *params, **kwargs).diag()
+        noise = self._shaped_noise_covar(function_samples.shape, *params, **kwargs).diagonal(dim1=-1, dim2=-2)
         return base_distributions.Normal(function_samples, noise.sqrt())
 
     def log_marginal(
@@ -266,7 +266,7 @@ class FixedNoiseGaussianLikelihood(_GaussianLikelihoodBase):
 
         if self.second_noise_covar is not None:
             res = res + self.second_noise_covar(*params, shape=shape, **kwargs)
-        elif isinstance(res, ZeroLazyTensor):
+        elif isinstance(res, ZeroLinearOperator):
             warnings.warn(
                 "You have passed data through a FixedNoiseGaussianLikelihood that did not match the size "
                 "of the fixed noise, *and* you did not specify noise. This is treated as a no-op.",
