@@ -1,43 +1,49 @@
 #!/usr/bin/env python3
 
-from typing import Any
+from typing import Any, Optional, Union
 
 import torch
+from linear_operator import to_linear_operator
+from linear_operator.operators import (
+    ConstantDiagLinearOperator,
+    DiagLinearOperator,
+    KroneckerProductDiagLinearOperator,
+    KroneckerProductLinearOperator,
+    LinearOperator,
+    RootLinearOperator,
+)
 from torch import Tensor
 
 from ..constraints import GreaterThan
-from ..distributions import base_distributions
-from ..lazy import (
-    ConstantDiagLazyTensor,
-    DiagLazyTensor,
-    KroneckerProductDiagLazyTensor,
-    KroneckerProductLazyTensor,
-    LazyEvaluatedKernelTensor,
-    RootLazyTensor,
-    lazify,
-)
+from ..distributions import MultitaskMultivariateNormal, base_distributions
+from ..lazy import LazyEvaluatedKernelTensor
 from ..likelihoods import Likelihood, _GaussianLikelihoodBase
+from ..module import Module
+from ..priors import Prior
 
 
 class _MultitaskGaussianLikelihoodBase(_GaussianLikelihoodBase):
-    """Base class for multi-task Gaussian Likelihoods, supporting general heteroskedastic noise models."""
+    r"""
+    Base class for multi-task Gaussian Likelihoods, supporting general heteroskedastic noise models.
 
-    def __init__(self, num_tasks, noise_covar, rank=0, task_correlation_prior=None, batch_shape=torch.Size()):
-        """
-        Args:
-            num_tasks (int):
-                Number of tasks.
-            noise_covar (:obj:`gpytorch.module.Module`):
-                A model for the noise covariance. This can be a simple homoskedastic noise model, or a GP
-                that is to be fitted on the observed measurement errors.
-            rank (int):
-                The rank of the task noise covariance matrix to fit. If `rank` is set to 0, then a diagonal covariance
-                matrix is fit.
-            task_correlation_prior (:obj:`gpytorch.priors.Prior`):
-                Prior to use over the task noise correlation matrix. Only used when `rank` > 0.
-            batch_shape (torch.Size):
-                Number of batches.
-        """
+    :param num_tasks: Number of tasks.
+    :param noise_covar: A model for the noise covariance. This can be a simple homoskedastic noise model, or a GP
+        that is to be fitted on the observed measurement errors.
+    :param rank: The rank of the task noise covariance matrix to fit. If `rank`
+        is set to 0, then a diagonal covariance matrix is fit.
+    :param task_correlation_prior: Prior to use over the task noise correlation
+        matrix. Only used when :math:`\text{rank} > 0`.
+    :param batch_shape: Number of batches.
+    """
+
+    def __init__(
+        self,
+        num_tasks: int,
+        noise_covar: Module,
+        rank: Optional[int] = 0,
+        task_correlation_prior: Optional[Prior] = None,
+        batch_shape: torch.Size = torch.Size(),
+    ):
         super().__init__(noise_covar=noise_covar)
         if rank != 0:
             if rank > num_tasks:
@@ -55,7 +61,7 @@ class _MultitaskGaussianLikelihoodBase(_GaussianLikelihoodBase):
         self.num_tasks = num_tasks
         self.rank = rank
 
-    def _eval_corr_matrix(self):
+    def _eval_corr_matrix(self) -> Tensor:
         tnc = self.task_noise_corr
         fac_diag = torch.ones(*tnc.shape[:-1], self.num_tasks, device=tnc.device, dtype=tnc.dtype)
         Cfac = torch.diag_embed(fac_diag)
@@ -64,28 +70,33 @@ class _MultitaskGaussianLikelihoodBase(_GaussianLikelihoodBase):
         C = Cfac / Cfac.pow(2).sum(dim=-1, keepdim=True).sqrt()
         return C @ C.transpose(-1, -2)
 
-    def marginal(self, function_dist, *params, **kwargs):
+    def marginal(self, function_dist: MultitaskMultivariateNormal, *params, **kwargs) -> MultitaskMultivariateNormal:
         r"""
-        If `rank` == 0, adds the task noises to the diagonal of the covariance matrix of the supplied
-        :obj:`gpytorch.distributions.MultivariateNormal` or :obj:`gpytorch.distributions.MultitaskMultivariateNormal`.
-        Otherwise, adds a rank `rank` covariance matrix to it.
+        If :math:`\text{rank} = 0`, adds the task noises to the diagonal of the
+        covariance matrix of the supplied
+        :obj:`~gpytorch.distributions.MultivariateNormal` or
+        :obj:`~gpytorch.distributions.MultitaskMultivariateNormal`.  Otherwise,
+        adds a rank `rank` covariance matrix to it.
 
-        To accomplish this, we form a new :obj:`gpytorch.lazy.KroneckerProductLazyTensor` between :math:`I_{n}`,
-        an identity matrix with size equal to the data and a (not necessarily diagonal) matrix containing the task
-        noises :math:`D_{t}`.
+        To accomplish this, we form a new
+        :obj:`~linear_operator.operators.KroneckerProductLinearOperator`
+        between :math:`I_{n}`, an identity matrix with size equal to the data
+        and a (not necessarily diagonal) matrix containing the task noises
+        :math:`D_{t}`.
 
         We also incorporate a shared `noise` parameter from the base
         :class:`gpytorch.likelihoods.GaussianLikelihood` that we extend.
 
-        The final covariance matrix after this method is then :math:`K + D_{t} \otimes I_{n} + \sigma^{2}I_{nt}`.
+        The final covariance matrix after this method is then
+        :math:`\mathbf K + \mathbf D_{t} \otimes \mathbf I_{n} + \sigma^{2} \mathbf I_{nt}`.
 
-        Args:
-            function_dist (:obj:`gpytorch.distributions.MultitaskMultivariateNormal`): Random variable whose covariance
-                matrix is a :obj:`gpytorch.lazy.LazyTensor` we intend to augment.
+        :param function_dist: Random variable whose covariance
+            matrix is a :obj:`~linear_operator.operators.LinearOperator` we intend to augment.
         Returns:
-            :obj:`gpytorch.distributions.MultitaskMultivariateNormal`: A new random variable whose covariance
-            matrix is a :obj:`gpytorch.lazy.LazyTensor` with :math:`D_{t} \otimes I_{n}` and :math:`\sigma^{2}I_{nt}`
-            added.
+            :obj:`gpytorch.distributions.MultitaskMultivariateNormal`:
+        :return: A new random variable whose covariance matrix is a
+            :obj:`~linear_operator.operators.LinearOperator` with
+            :math:`\mathbf D_{t} \otimes \mathbf I_{n}` and :math:`\sigma^{2} \mathbf I_{nt}` added.
         """
         mean, covar = function_dist.mean, function_dist.lazy_covariance_matrix
 
@@ -98,23 +109,27 @@ class _MultitaskGaussianLikelihoodBase(_GaussianLikelihoodBase):
 
         return function_dist.__class__(mean, covar)
 
-    def _shaped_noise_covar(self, shape, add_noise=True, *params, **kwargs):
+    def _shaped_noise_covar(
+        self, shape: torch.Size, add_noise: Optional[bool] = True, *params, **kwargs
+    ) -> LinearOperator:
         if not self.has_task_noise:
-            noise = ConstantDiagLazyTensor(self.noise, diag_shape=shape[-2] * self.num_tasks)
+            noise = ConstantDiagLinearOperator(self.noise, diag_shape=shape[-2] * self.num_tasks)
             return noise
 
         if self.rank == 0:
             task_noises = self.raw_task_noises_constraint.transform(self.raw_task_noises)
-            task_var_lt = DiagLazyTensor(task_noises)
+            task_var_lt = DiagLinearOperator(task_noises)
             dtype, device = task_noises.dtype, task_noises.device
-            ckl_init = KroneckerProductDiagLazyTensor
+            ckl_init = KroneckerProductDiagLinearOperator
         else:
             task_noise_covar_factor = self.task_noise_covar_factor
-            task_var_lt = RootLazyTensor(task_noise_covar_factor)
+            task_var_lt = RootLinearOperator(task_noise_covar_factor)
             dtype, device = task_noise_covar_factor.dtype, task_noise_covar_factor.device
-            ckl_init = KroneckerProductLazyTensor
+            ckl_init = KroneckerProductLinearOperator
 
-        eye_lt = ConstantDiagLazyTensor(torch.ones(*shape[:-2], 1, dtype=dtype, device=device), diag_shape=shape[-2])
+        eye_lt = ConstantDiagLinearOperator(
+            torch.ones(*shape[:-2], 1, dtype=dtype, device=device), diag_shape=shape[-2]
+        )
         task_var_lt = task_var_lt.expand(*shape[:-2], *task_var_lt.matrix_shape)
 
         # to add the latent noise we exploit the fact that
@@ -122,7 +137,7 @@ class _MultitaskGaussianLikelihoodBase(_GaussianLikelihoodBase):
         # which allows us to move the latent noise inside the task dependent noise
         # thereby allowing exploitation of Kronecker structure in this likelihood.
         if add_noise and self.has_global_noise:
-            noise = ConstantDiagLazyTensor(self.noise, diag_shape=task_var_lt.shape[-1])
+            noise = ConstantDiagLinearOperator(self.noise, diag_shape=task_var_lt.shape[-1])
             task_var_lt = task_var_lt + noise
 
         covar_kron_lt = ckl_init(eye_lt, task_var_lt)
@@ -130,20 +145,34 @@ class _MultitaskGaussianLikelihoodBase(_GaussianLikelihoodBase):
         return covar_kron_lt
 
     def forward(self, function_samples: Tensor, *params: Any, **kwargs: Any) -> base_distributions.Normal:
-        noise = self._shaped_noise_covar(function_samples.shape, *params, **kwargs).diag()
+        noise = self._shaped_noise_covar(function_samples.shape, *params, **kwargs).diagonal(dim1=-1, dim2=-2)
         noise = noise.reshape(*noise.shape[:-1], *function_samples.shape[-2:])
         return base_distributions.Independent(base_distributions.Normal(function_samples, noise.sqrt()), 1)
 
 
 class MultitaskGaussianLikelihood(_MultitaskGaussianLikelihoodBase):
-    """
-    A convenient extension of the :class:`gpytorch.likelihoods.GaussianLikelihood` to the multitask setting that allows
+    r"""
+    A convenient extension of the :class:`~gpytorch.likelihoods.GaussianLikelihood` to the multitask setting that allows
     for a full cross-task covariance structure for the noise. The fitted covariance matrix has rank `rank`.
     If a strictly diagonal task noise covariance matrix is desired, then rank=0 should be set. (This option still
     allows for a different `noise` parameter for each task.)
 
     Like the Gaussian likelihood, this object can be used with exact inference.
 
+    .. note::
+        At least one of :attr:`has_global_noise` or :attr:`has_task_noise` should be specified.
+
+    :param num_tasks: Number of tasks.
+    :param noise_covar: A model for the noise covariance. This can be a simple homoskedastic noise model, or a GP
+        that is to be fitted on the observed measurement errors.
+    :param rank: The rank of the task noise covariance matrix to fit. If `rank`
+        is set to 0, then a diagonal covariance matrix is fit.
+    :param task_prior: Prior to use over the task noise correlation
+        matrix. Only used when :math:`\text{rank} > 0`.
+    :param batch_shape: Number of batches.
+    :param has_global_noise: Whether to include a :math:`\sigma^2 \mathbf I_{nt}` term in the noise model.
+    :param has_task_noise: Whether to include task-specific noise terms, which add
+        :math:`\mathbf I_n \otimes \mathbf D_T` into the noise model.
     """
 
     def __init__(
@@ -157,24 +186,6 @@ class MultitaskGaussianLikelihood(_MultitaskGaussianLikelihoodBase):
         has_global_noise=True,
         has_task_noise=True,
     ):
-        """
-        Args:
-            num_tasks (int): Number of tasks.
-
-            rank (int): The rank of the task noise covariance matrix to fit. If `rank` is set to 0,
-            then a diagonal covariance matrix is fit.
-
-            task_prior (:obj:`gpytorch.priors.Prior`): Prior to use over the task noise covariance matrix if
-            `rank` > 0, or a prior over the log of just the diagonal elements, if `rank` == 0.
-
-            has_global_noise (bool): whether to include a \\sigma^2 I_{nt} term in the noise model.
-
-            has_task_noise (bool): whether to include task-specific noise terms, which add I_n \kron D_T
-            into the noise model.
-
-            At least one of has_global_noise or has_task_noise should be specified.
-
-        """
         super(Likelihood, self).__init__()
         if noise_constraint is None:
             noise_constraint = GreaterThan(1e-4)
@@ -215,51 +226,51 @@ class MultitaskGaussianLikelihood(_MultitaskGaussianLikelihoodBase):
         self.has_task_noise = has_task_noise
 
     @property
-    def noise(self):
+    def noise(self) -> Tensor:
         return self.raw_noise_constraint.transform(self.raw_noise)
 
     @noise.setter
-    def noise(self, value):
+    def noise(self, value: Union[float, Tensor]):
         self._set_noise(value)
 
     @property
-    def task_noises(self):
+    def task_noises(self) -> Tensor:
         if self.rank == 0:
             return self.raw_task_noises_constraint.transform(self.raw_task_noises)
         else:
             raise AttributeError("Cannot set diagonal task noises when covariance has ", self.rank, ">0")
 
     @task_noises.setter
-    def task_noises(self, value):
+    def task_noises(self, value: Union[float, Tensor]):
         if self.rank == 0:
             self._set_task_noises(value)
         else:
             raise AttributeError("Cannot set diagonal task noises when covariance has ", self.rank, ">0")
 
-    def _set_noise(self, value):
+    def _set_noise(self, value: Union[float, Tensor]):
         self.initialize(raw_noise=self.raw_noise_constraint.inverse_transform(value))
 
-    def _set_task_noises(self, value):
+    def _set_task_noises(self, value: Union[float, Tensor]):
         self.initialize(raw_task_noises=self.raw_task_noises_constraint.inverse_transform(value))
 
     @property
-    def task_noise_covar(self):
+    def task_noise_covar(self) -> Tensor:
         if self.rank > 0:
             return self.task_noise_covar_factor.matmul(self.task_noise_covar_factor.transpose(-1, -2))
         else:
             raise AttributeError("Cannot retrieve task noises when covariance is diagonal.")
 
     @task_noise_covar.setter
-    def task_noise_covar(self, value):
+    def task_noise_covar(self, value: Union[float, Tensor]):
         # internally uses a pivoted cholesky decomposition to construct a low rank
         # approximation of the covariance
         if self.rank > 0:
             with torch.no_grad():
-                self.task_noise_covar_factor.data = lazify(value).pivoted_cholesky(rank=self.rank)
+                self.task_noise_covar_factor.data = to_linear_operator(value).pivoted_cholesky(rank=self.rank)
         else:
             raise AttributeError("Cannot set non-diagonal task noises when covariance is diagonal.")
 
-    def _eval_covar_matrix(self):
+    def _eval_covar_matrix(self) -> Tensor:
         covar_factor = self.task_noise_covar_factor
         noise = self.noise
         D = noise * torch.eye(self.num_tasks, dtype=noise.dtype, device=noise.device)
