@@ -4,24 +4,25 @@ import functools
 import string
 
 import torch
+from linear_operator import to_dense, to_linear_operator
+from linear_operator.operators import (
+    AddedDiagLinearOperator,
+    BatchRepeatLinearOperator,
+    ConstantMulLinearOperator,
+    DenseLinearOperator,
+    InterpolatedLinearOperator,
+    LinearOperator,
+    LowRankRootAddedDiagLinearOperator,
+    MatmulLinearOperator,
+    RootLinearOperator,
+    ZeroLinearOperator,
+)
+from linear_operator.utils.interpolation import left_interp, left_t_interp
+from torch import Tensor
 
 from .. import settings
-from ..lazy import (
-    AddedDiagLazyTensor,
-    BatchRepeatLazyTensor,
-    ConstantMulLazyTensor,
-    InterpolatedLazyTensor,
-    LazyEvaluatedKernelTensor,
-    LowRankRootAddedDiagLazyTensor,
-    MatmulLazyTensor,
-    NonLazyTensor,
-    RootLazyTensor,
-    ZeroLazyTensor,
-    delazify,
-    lazify,
-)
+from ..lazy import LazyEvaluatedKernelTensor
 from ..utils.cholesky import psd_safe_cholesky
-from ..utils.interpolation import left_interp, left_t_interp
 from ..utils.memoize import add_to_cache, cached, clear_cache_hook, pop_from_cache
 
 
@@ -61,10 +62,10 @@ class DefaultPredictionStrategy(object):
         self.lik_train_train_covar = mvn.lazy_covariance_matrix
 
         if root is not None:
-            add_to_cache(self.lik_train_train_covar, "root_decomposition", RootLazyTensor(root))
+            add_to_cache(self.lik_train_train_covar, "root_decomposition", RootLinearOperator(root))
 
         if inv_root is not None:
-            add_to_cache(self.lik_train_train_covar, "root_inv_decomposition", RootLazyTensor(inv_root))
+            add_to_cache(self.lik_train_train_covar, "root_inv_decomposition", RootLinearOperator(inv_root))
 
     def __deepcopy__(self, memo):
         # deepcopying prediction strategies of a model evaluated on inputs that require gradients fails
@@ -106,7 +107,7 @@ class DefaultPredictionStrategy(object):
             test_train_covar (:obj:`torch.tensor`): The observed noise (from the likelihood)
 
         Returns
-            :obj:`~gpytorch.lazy.LazyTensor`: :math:`K_{X^{*}X} S`
+            :obj:`~linear_operator.operators.LinearOperator`: :math:`K_{X^{*}X} S`
         """
         # Here the precomputed cache represents S,
         # where S S^T = (K_XX + sigma^2 I)^-1
@@ -148,7 +149,7 @@ class DefaultPredictionStrategy(object):
         mvn_obs = fant_likelihood(mvn, inputs, **kwargs)
 
         fant_fant_covar = mvn_obs.covariance_matrix
-        fant_train_covar = delazify(full_covar[..., num_train:, :num_train])
+        fant_train_covar = to_dense(full_covar[..., num_train:, :num_train])
 
         self.fantasy_inputs = inputs
         self.fantasy_targets = targets
@@ -194,8 +195,8 @@ class DefaultPredictionStrategy(object):
 
         # now update the root and root inverse
         new_lt = self.lik_train_train_covar.cat_rows(fant_train_covar, fant_fant_covar)
-        new_root = new_lt.root_decomposition().root.evaluate()
-        new_covar_cache = new_lt.root_inv_decomposition().root.evaluate()
+        new_root = new_lt.root_decomposition().root.to_dense()
+        new_covar_cache = new_lt.root_inv_decomposition().root.to_dense()
 
         # Expand inputs accordingly if necessary (for fantasies at the same points)
         if full_inputs[0].dim() <= full_targets.dim():
@@ -204,8 +205,8 @@ class DefaultPredictionStrategy(object):
             repeat_shape = fant_batch_shape + torch.Size([1] * n_batch)
             full_inputs = [fi.expand(fant_batch_shape + fi.shape) for fi in full_inputs]
             full_mean = full_mean.expand(fant_batch_shape + full_mean.shape)
-            full_covar = BatchRepeatLazyTensor(full_covar, repeat_shape)
-            new_root = BatchRepeatLazyTensor(NonLazyTensor(new_root), repeat_shape)
+            full_covar = BatchRepeatLinearOperator(full_covar, repeat_shape)
+            new_root = BatchRepeatLinearOperator(DenseLinearOperator(new_root), repeat_shape)
             # no need to repeat the covar cache, broadcasting will do the right thing
 
         # Create new DefaultPredictionStrategy object
@@ -225,7 +226,7 @@ class DefaultPredictionStrategy(object):
     @cached(name="covar_cache")
     def covar_cache(self):
         train_train_covar = self.lik_train_train_covar
-        train_train_covar_inv_root = delazify(train_train_covar.root_inv_decomposition().root)
+        train_train_covar_inv_root = to_dense(train_train_covar.root_inv_decomposition().root)
         return self._exact_predictive_covar_inv_quad_form_cache(train_train_covar_inv_root, self._last_test_train_covar)
 
     @property
@@ -235,7 +236,7 @@ class DefaultPredictionStrategy(object):
         train_mean, train_train_covar = mvn.loc, mvn.lazy_covariance_matrix
 
         train_labels_offset = (self.train_labels - train_mean).unsqueeze(-1)
-        mean_cache = train_train_covar.evaluate_kernel().inv_matmul(train_labels_offset).squeeze(-1)
+        mean_cache = train_train_covar.evaluate_kernel().solve(train_labels_offset).squeeze(-1)
 
         if settings.detach_test_caches.on():
             mean_cache = mean_cache.detach()
@@ -260,7 +261,7 @@ class DefaultPredictionStrategy(object):
         test_mean = joint_mean[..., self.num_train :]
         # For efficiency - we can make things more efficient
         if joint_covar.size(-1) <= settings.max_eager_kernel_size.value():
-            test_covar = joint_covar[..., self.num_train :, :].evaluate()
+            test_covar = joint_covar[..., self.num_train :, :].to_dense()
             test_test_covar = test_covar[..., self.num_train :]
             test_train_covar = test_covar[..., : self.num_train]
         else:
@@ -272,42 +273,39 @@ class DefaultPredictionStrategy(object):
             self.exact_predictive_covar(test_test_covar, test_train_covar),
         )
 
-    def exact_predictive_mean(self, test_mean, test_train_covar):
+    def exact_predictive_mean(self, test_mean: Tensor, test_train_covar: LinearOperator) -> Tensor:
         """
         Computes the posterior predictive covariance of a GP
 
-        Args:
-            test_mean (:obj:`torch.tensor`): The test prior mean
-            test_train_covar (:obj:`gpytorch.lazy.LazyTensor`): Covariance matrix between test and train inputs
-
-        Returns:
-            :obj:`torch.tensor`: The predictive posterior mean of the test points
+        :param Tensor test_mean: The test prior mean
+        :param ~linear_operator.operators.LinearOperator test_train_covar:
+            Covariance matrix between test and train inputs
+        :return: The predictive posterior mean of the test points
         """
         # NOTE TO FUTURE SELF:
         # You **cannot* use addmv here, because test_train_covar may not actually be a non lazy tensor even for an exact
-        # GP, and using addmv requires you to delazify test_train_covar, which is obviously a huge no-no!
+        # GP, and using addmv requires you to to_dense test_train_covar, which is obviously a huge no-no!
         res = (test_train_covar @ self.mean_cache.unsqueeze(-1)).squeeze(-1)
         res = res + test_mean
 
         return res
 
-    def exact_predictive_covar(self, test_test_covar, test_train_covar):
+    def exact_predictive_covar(
+        self, test_test_covar: LinearOperator, test_train_covar: LinearOperator
+    ) -> LinearOperator:
         """
         Computes the posterior predictive covariance of a GP
 
-        Args:
-            test_train_covar (:obj:`gpytorch.lazy.LazyTensor`): Covariance matrix between test and train inputs
-            test_test_covar (:obj:`gpytorch.lazy.LazyTensor`): Covariance matrix between test inputs
-
-        Returns:
-            :obj:`gpytorch.lazy.LazyTensor`: A LazyTensor representing the predictive posterior covariance of the
-                                               test points
+        :param ~linear_operator.operators.LinearOperator test_train_covar:
+            Covariance matrix between test and train inputs
+        :param ~linear_operator.operators.LinearOperator test_test_covar: Covariance matrix between test inputs
+        :return: A LinearOperator representing the predictive posterior covariance of the test points
         """
         if settings.fast_pred_var.on():
             self._last_test_train_covar = test_train_covar
 
         if settings.skip_posterior_variances.on():
-            return ZeroLazyTensor(*test_test_covar.size())
+            return ZeroLinearOperator(*test_test_covar.size())
 
         if settings.fast_pred_var.off():
             dist = self.train_prior_dist.__class__(
@@ -318,32 +316,32 @@ class DefaultPredictionStrategy(object):
             else:
                 train_train_covar = self.likelihood(dist, self.train_inputs).lazy_covariance_matrix
 
-            test_train_covar = delazify(test_train_covar)
+            test_train_covar = to_dense(test_train_covar)
             train_test_covar = test_train_covar.transpose(-1, -2)
-            covar_correction_rhs = train_train_covar.inv_matmul(train_test_covar)
+            covar_correction_rhs = train_train_covar.solve(train_test_covar)
             # For efficiency
             if torch.is_tensor(test_test_covar):
                 # We can use addmm in the 2d case
                 if test_test_covar.dim() == 2:
-                    return lazify(
+                    return to_linear_operator(
                         torch.addmm(test_test_covar, test_train_covar, covar_correction_rhs, beta=1, alpha=-1)
                     )
                 else:
-                    return lazify(test_test_covar + test_train_covar @ covar_correction_rhs.mul(-1))
+                    return to_linear_operator(test_test_covar + test_train_covar @ covar_correction_rhs.mul(-1))
             # In other cases - we'll use the standard infrastructure
             else:
-                return test_test_covar + MatmulLazyTensor(test_train_covar, covar_correction_rhs.mul(-1))
+                return test_test_covar + MatmulLinearOperator(test_train_covar, covar_correction_rhs.mul(-1))
 
         precomputed_cache = self.covar_cache
         covar_inv_quad_form_root = self._exact_predictive_covar_inv_quad_form_root(precomputed_cache, test_train_covar)
         if torch.is_tensor(test_test_covar):
-            return lazify(
+            return to_linear_operator(
                 torch.add(
                     test_test_covar, covar_inv_quad_form_root @ covar_inv_quad_form_root.transpose(-1, -2), alpha=-1
                 )
             )
         else:
-            return test_test_covar + MatmulLazyTensor(
+            return test_test_covar + MatmulLinearOperator(
                 covar_inv_quad_form_root, covar_inv_quad_form_root.transpose(-1, -2).mul(-1)
             )
 
@@ -365,9 +363,9 @@ class InterpolatedPredictionStrategy(DefaultPredictionStrategy):
     def _exact_predictive_covar_inv_quad_form_cache(self, train_train_covar_inv_root, test_train_covar):
         train_interp_indices = test_train_covar.right_interp_indices
         train_interp_values = test_train_covar.right_interp_values
-        base_lazy_tensor = test_train_covar.base_lazy_tensor
-        base_size = base_lazy_tensor.size(-1)
-        res = base_lazy_tensor.matmul(
+        base_linear_op = test_train_covar.base_linear_op
+        base_size = base_linear_op.size(-1)
+        res = base_linear_op.matmul(
             left_t_interp(train_interp_indices, train_interp_values, train_train_covar_inv_root, base_size)
         )
         return res
@@ -401,9 +399,9 @@ class InterpolatedPredictionStrategy(DefaultPredictionStrategy):
         fant_noise = fant_likelihood.noise_covar(fant_wmat.transpose(-1, -2) if len(fant_wmat.shape) > 2 else fant_wmat)
         fant_root_vector = fant_noise.sqrt_inv_matmul(fant_wmat.transpose(-1, -2)).transpose(-1, -2)
 
-        new_wmat = self.interp_inner_prod.add_low_rank(fant_root_vector.evaluate())
+        new_wmat = self.interp_inner_prod.add_low_rank(fant_root_vector.to_dense())
         mean_diff = (targets - fant_mean).unsqueeze(-1)
-        new_interp_response_cache = self.interp_response_cache + fant_wmat.matmul(fant_noise.inv_matmul(mean_diff))
+        new_interp_response_cache = self.interp_response_cache + fant_wmat.matmul(fant_noise.solve(mean_diff))
 
         # Create new DefaultPredictionStrategy object
         fant_strat = self.__class__(
@@ -422,7 +420,7 @@ class InterpolatedPredictionStrategy(DefaultPredictionStrategy):
         if covar is None:
             covar = self.train_prior_dist.lazy_covariance_matrix
         wmat = covar._sparse_left_interp_t(covar.left_interp_indices, covar.left_interp_values).to_dense()
-        return lazify(wmat)
+        return to_linear_operator(wmat)
 
     @property
     @cached(name="interp_inner_prod")
@@ -430,7 +428,7 @@ class InterpolatedPredictionStrategy(DefaultPredictionStrategy):
         # the W'W cache
         wmat = self.prepare_dense_wmat()
         noise_term = self.likelihood.noise_covar(wmat.transpose(-1, -2) if len(wmat.shape) > 2 else wmat)
-        interp_inner_prod = wmat.matmul(noise_term.inv_matmul(wmat.transpose(-1, -2)))
+        interp_inner_prod = wmat.matmul(noise_term.solve(wmat.transpose(-1, -2)))
         return interp_inner_prod
 
     @property
@@ -439,7 +437,7 @@ class InterpolatedPredictionStrategy(DefaultPredictionStrategy):
         wmat = self.prepare_dense_wmat()
         noise_term = self.likelihood.noise_covar(wmat.transpose(-1, -2) if len(wmat.shape) > 2 else wmat)
         demeaned_train_targets = self.train_labels - self.train_prior_dist.mean
-        dinv_y = noise_term.inv_matmul(demeaned_train_targets.unsqueeze(-1))
+        dinv_y = noise_term.solve(demeaned_train_targets.unsqueeze(-1))
         return wmat.matmul(dinv_y)
 
     @property
@@ -453,11 +451,11 @@ class InterpolatedPredictionStrategy(DefaultPredictionStrategy):
         train_mean, train_train_covar_with_noise = mvn.mean, mvn.lazy_covariance_matrix
 
         mean_diff = (self.train_labels - train_mean).unsqueeze(-1)
-        train_train_covar_inv_labels = train_train_covar_with_noise.inv_matmul(mean_diff)
+        train_train_covar_inv_labels = train_train_covar_with_noise.solve(mean_diff)
 
         # New root factor
-        base_size = train_train_covar.base_lazy_tensor.size(-1)
-        mean_cache = train_train_covar.base_lazy_tensor.matmul(
+        base_size = train_train_covar.base_linear_op.size(-1)
+        mean_cache = train_train_covar.base_linear_op.matmul(
             left_t_interp(train_interp_indices, train_interp_values, train_train_covar_inv_labels, base_size)
         )
 
@@ -472,7 +470,7 @@ class InterpolatedPredictionStrategy(DefaultPredictionStrategy):
     def fantasy_mean_cache(self):
         # first construct K_UU
         train_train_covar = self.train_prior_dist.lazy_covariance_matrix
-        inducing_covar = train_train_covar.base_lazy_tensor
+        inducing_covar = train_train_covar.base_linear_op
 
         # now get L such that LL' \approx WD^{-1}W'
         interp_inner_prod_root = self.interp_inner_prod.root_decomposition(method="cholesky").root
@@ -488,7 +486,7 @@ class InterpolatedPredictionStrategy(DefaultPredictionStrategy):
         # L' m
         root_space_projection = interp_inner_prod_root.transpose(-1, -2).matmul(inducing_covar_response)
         # Q^{-1} (L' m)
-        qmat_solve = current_qmatrix.inv_matmul(root_space_projection)
+        qmat_solve = current_qmatrix.solve(root_space_projection)
 
         mean_cache = inducing_covar_response - inducing_compression_matrix @ qmat_solve
 
@@ -502,7 +500,7 @@ class InterpolatedPredictionStrategy(DefaultPredictionStrategy):
     @cached(name="fantasy_covar_cache")
     def fantasy_covar_cache(self):
         train_train_covar = self.train_prior_dist.lazy_covariance_matrix
-        inducing_covar = train_train_covar.base_lazy_tensor
+        inducing_covar = train_train_covar.base_linear_op
 
         # we need to enforce a cholesky here for numerical stability
         interp_inner_prod_root = self.interp_inner_prod.root_decomposition(method="cholesky").root
@@ -512,12 +510,12 @@ class InterpolatedPredictionStrategy(DefaultPredictionStrategy):
 
         if settings.fast_pred_var.on():
             qmat_inv_root = current_qmatrix.root_inv_decomposition()
-            # to lazify you have to evaluate the inverse root which is slow
+            # to to_linear_operator you have to evaluate the inverse root which is slow
             # otherwise, you can't backprop your way through it
-            inner_cache = RootLazyTensor(inducing_compression_matrix.matmul(qmat_inv_root.root.evaluate()))
+            inner_cache = RootLinearOperator(inducing_compression_matrix.matmul(qmat_inv_root.root.to_dense()))
         else:
             inner_cache = inducing_compression_matrix.matmul(
-                current_qmatrix.inv_matmul(inducing_compression_matrix.transpose(-1, -2))
+                current_qmatrix.solve(inducing_compression_matrix.transpose(-1, -2))
             )
 
         # Precomputed factor
@@ -548,7 +546,7 @@ class InterpolatedPredictionStrategy(DefaultPredictionStrategy):
 
         # Get probe vectors for inverse root
         num_probe_vectors = settings.fast_pred_var.num_probe_vectors()
-        num_inducing = train_train_covar.base_lazy_tensor.size(-1)
+        num_inducing = train_train_covar.base_linear_op.size(-1)
         vector_indices = torch.randperm(num_inducing).type_as(train_interp_indices)
         probe_vector_indices = vector_indices[:num_probe_vectors]
         test_vector_indices = vector_indices[num_probe_vectors : 2 * num_probe_vectors]
@@ -559,21 +557,21 @@ class InterpolatedPredictionStrategy(DefaultPredictionStrategy):
         device = train_train_covar.device
         probe_interp_values = torch.ones(num_probe_vectors, 1, dtype=dtype, device=device)
 
-        batch_shape = train_train_covar.base_lazy_tensor.batch_shape
-        probe_vectors = InterpolatedLazyTensor(
-            train_train_covar.base_lazy_tensor,
+        batch_shape = train_train_covar.base_linear_op.batch_shape
+        probe_vectors = InterpolatedLinearOperator(
+            train_train_covar.base_linear_op,
             train_interp_indices.expand(*batch_shape, *train_interp_indices.shape[-2:]),
             train_interp_values.expand(*batch_shape, *train_interp_values.shape[-2:]),
             probe_interp_indices.expand(*batch_shape, *probe_interp_indices.shape[-2:]),
             probe_interp_values.expand(*batch_shape, *probe_interp_values.shape[-2:]),
-        ).evaluate()
-        test_vectors = InterpolatedLazyTensor(
-            train_train_covar.base_lazy_tensor,
+        ).to_dense()
+        test_vectors = InterpolatedLinearOperator(
+            train_train_covar.base_linear_op,
             train_interp_indices.expand(*batch_shape, *train_interp_indices.shape[-2:]),
             train_interp_values.expand(*batch_shape, *train_interp_values.shape[-2:]),
             probe_test_interp_indices.expand(*batch_shape, *probe_test_interp_indices.shape[-2:]),
             probe_interp_values.expand(*batch_shape, *probe_interp_values.shape[-2:]),
-        ).evaluate()
+        ).to_dense()
 
         # Put data through the likelihood
         dist = self.train_prior_dist.__class__(
@@ -585,15 +583,15 @@ class InterpolatedPredictionStrategy(DefaultPredictionStrategy):
         train_train_covar_inv_root = train_train_covar_plus_noise.root_inv_decomposition(
             initial_vectors=probe_vectors, test_vectors=test_vectors
         ).root
-        train_train_covar_inv_root = train_train_covar_inv_root.evaluate()
+        train_train_covar_inv_root = train_train_covar_inv_root.to_dense()
 
         # New root factor
         root = self._exact_predictive_covar_inv_quad_form_cache(train_train_covar_inv_root, self._last_test_train_covar)
 
         # Precomputed factor
         if settings.fast_pred_samples.on():
-            inside = train_train_covar.base_lazy_tensor + RootLazyTensor(root).mul(-1)
-            inside_root = inside.root_decomposition().root.evaluate()
+            inside = train_train_covar.base_linear_op + RootLinearOperator(root).mul(-1)
+            inside_root = inside.root_decomposition().root.to_dense()
             # Prevent backprop through this variable
             if settings.detach_test_caches.on():
                 inside_root = inside_root.detach()
@@ -636,11 +634,11 @@ class InterpolatedPredictionStrategy(DefaultPredictionStrategy):
             precomputed_cache = self.fantasy_covar_cache
             fps = settings.fast_pred_samples.on()
             if fps:
-                root = left_interp(test_interp_indices, test_interp_values, precomputed_cache[0].evaluate())
-                res = RootLazyTensor(root)
+                root = left_interp(test_interp_indices, test_interp_values, precomputed_cache[0].to_dense())
+                res = RootLinearOperator(root)
             else:
-                root = left_interp(test_interp_indices, test_interp_values, precomputed_cache[1].evaluate())
-                res = test_test_covar + RootLazyTensor(root).mul(-1)
+                root = left_interp(test_interp_indices, test_interp_values, precomputed_cache[1].to_dense())
+                res = test_test_covar + RootLinearOperator(root).mul(-1)
             return res
         else:
             precomputed_cache = self.covar_cache
@@ -652,10 +650,10 @@ class InterpolatedPredictionStrategy(DefaultPredictionStrategy):
             # Compute the exact predictive posterior
             if settings.fast_pred_samples.on():
                 res = self._exact_predictive_covar_inv_quad_form_root(precomputed_cache[0], test_train_covar)
-                res = RootLazyTensor(res)
+                res = RootLinearOperator(res)
             else:
                 root = left_interp(test_interp_indices, test_interp_values, precomputed_cache[1])
-                res = test_test_covar + RootLazyTensor(root).mul(-1)
+                res = test_test_covar + RootLinearOperator(root).mul(-1)
             return res
 
 
@@ -673,17 +671,17 @@ class RFFPredictionStrategy(DefaultPredictionStrategy):
     @cached(name="covar_cache")
     def covar_cache(self):
         lt = self.train_prior_dist.lazy_covariance_matrix
-        if isinstance(lt, ConstantMulLazyTensor):
+        if isinstance(lt, ConstantMulLinearOperator):
             constant = lt.expanded_constant
-            lt = lt.base_lazy_tensor
+            lt = lt.base_linear_op
         else:
             constant = torch.tensor(1.0, dtype=lt.dtype, device=lt.device)
 
-        train_factor = lt.root.evaluate()
+        train_factor = lt.root.to_dense()
         train_train_covar = self.lik_train_train_covar
         inner_term = (
             torch.eye(train_factor.size(-1), dtype=train_factor.dtype, device=train_factor.device)
-            - (train_factor.transpose(-1, -2) @ train_train_covar.inv_matmul(train_factor)) * constant
+            - (train_factor.transpose(-1, -2) @ train_train_covar.solve(train_factor)) * constant
         )
         return psd_safe_cholesky(inner_term)
 
@@ -700,17 +698,17 @@ class RFFPredictionStrategy(DefaultPredictionStrategy):
 
     def exact_predictive_covar(self, test_test_covar, test_train_covar):
         if settings.skip_posterior_variances.on():
-            return ZeroLazyTensor(*test_test_covar.size())
+            return ZeroLinearOperator(*test_test_covar.size())
 
-        if isinstance(test_test_covar, ConstantMulLazyTensor):
+        if isinstance(test_test_covar, ConstantMulLinearOperator):
             constant = test_test_covar.expanded_constant
-            test_test_covar = test_test_covar.base_lazy_tensor
+            test_test_covar = test_test_covar.base_linear_op
         else:
             constant = torch.tensor(1.0, dtype=test_test_covar.dtype, device=test_test_covar.device)
 
         covar_cache = self.covar_cache
-        factor = test_test_covar.root.evaluate() * constant.sqrt()
-        res = RootLazyTensor(factor @ covar_cache)
+        factor = test_test_covar.root.to_dense() * constant.sqrt()
+        res = RootLinearOperator(factor @ covar_cache)
         return res
 
 
@@ -725,18 +723,22 @@ class SGPRPredictionStrategy(DefaultPredictionStrategy):
         train_train_covar = self.lik_train_train_covar.evaluate_kernel()
 
         # Get terms needed for woodbury
-        root = train_train_covar._lazy_tensor.root_decomposition().root.evaluate()  # R
+        root = train_train_covar._linear_op.root_decomposition().root.to_dense()  # R
         inv_diag = train_train_covar._diag_tensor.inverse()  # \sigma^{-2}
 
         # Form LT using woodbury
         ones = torch.tensor(1.0, dtype=root.dtype, device=root.device)
-        chol_factor = lazify(root.transpose(-1, -2) @ (inv_diag @ root)).add_diag(ones)  # (I + \sigma^{-2} R^T R)^{-1}
-        woodbury_term = inv_diag @ torch.triangular_solve(
-            root.transpose(-1, -2), chol_factor.cholesky().evaluate(), upper=False
-        )[0].transpose(-1, -2)
+        chol_factor = to_linear_operator(root.transpose(-1, -2) @ (inv_diag @ root)).add_diagonal(
+            ones
+        )  # (I + \sigma^{-2} R^T R)^{-1}
+        woodbury_term = inv_diag @ torch.linalg.solve_triangular(
+            chol_factor.cholesky().to_dense(), root.transpose(-1, -2), upper=False
+        ).transpose(-1, -2)
         # woodbury_term @ woodbury_term^T = \sigma^{-2} R (I + \sigma^{-2} R^T R)^{-1} R^T \sigma^{-2}
 
-        inverse = AddedDiagLazyTensor(inv_diag, MatmulLazyTensor(-woodbury_term, woodbury_term.transpose(-1, -2)))
+        inverse = AddedDiagLinearOperator(
+            inv_diag, MatmulLinearOperator(-woodbury_term, woodbury_term.transpose(-1, -2))
+        )
         # \sigma^{-2} ( I - \sigma^{-2} R (I + \sigma^{-2} R^T R)^{-1} R^T  )
 
         return root.transpose(-1, -2) @ (inverse @ root)
@@ -777,16 +779,16 @@ class SGPRPredictionStrategy(DefaultPredictionStrategy):
         # covar_cache = K_{UU}^{-1/2} K_{UX}( K_{XX} + \sigma^2 I )^{-1} K_{XU} K_{UU}^{-1/2}
 
         # Decompose test_train_covar = l, r
-        # Main case: test_x and train_x are different - test_train_covar is a MatmulLazyTensor
-        if isinstance(test_train_covar, MatmulLazyTensor):
-            L = test_train_covar.left_lazy_tensor.evaluate()
-        # Edge case: test_x and train_x are the same - test_train_covar is a LowRankRootAddedDiagLazyTensor
-        elif isinstance(test_train_covar, LowRankRootAddedDiagLazyTensor):
-            L = test_train_covar._lazy_tensor.root.evaluate()
+        # Main case: test_x and train_x are different - test_train_covar is a MatmulLinearOperator
+        if isinstance(test_train_covar, MatmulLinearOperator):
+            L = test_train_covar.left_linear_op.to_dense()
+        # Edge case: test_x and train_x are the same - test_train_covar is a LowRankRootAddedDiagLinearOperator
+        elif isinstance(test_train_covar, LowRankRootAddedDiagLinearOperator):
+            L = test_train_covar._linear_op.root.to_dense()
         else:
             # We should not hit this point of the code - this is to catch potential bugs in GPyTorch
             raise ValueError(
-                "Expected SGPR output to be a MatmulLazyTensor or AddedDiagLazyTensor. "
+                "Expected SGPR output to be a MatmulLinearOperator or AddedDiagLinearOperator. "
                 f"Got {test_train_covar.__class__.__name__} instead. "
                 "This is likely a bug in GPyTorch."
             )
