@@ -3,13 +3,13 @@
 from typing import Optional, Tuple
 
 import torch
+from linear_operator import to_linear_operator
+from linear_operator.operators import DiagLinearOperator, MatmulLinearOperator, SumLinearOperator
+from linear_operator.utils import linear_cg
 
 from .. import settings
 from ..distributions import Delta, MultivariateNormal
-from ..lazy import DiagLazyTensor, MatmulLazyTensor, SumLazyTensor, lazify
 from ..module import Module
-from ..utils import linear_cg
-from ..utils.broadcasting import _mul_broadcast_shape
 from ..utils.memoize import cached
 from ._variational_strategy import _VariationalStrategy
 from .natural_variational_distribution import NaturalVariationalDistribution
@@ -41,7 +41,7 @@ class _NgdInterpTerms(torch.autograd.Function):
         diag = prec.diagonal(dim1=-1, dim2=-2).unsqueeze(-1)
 
         # Make sure that interp_term and natural_vec are the same batch shape
-        batch_shape = _mul_broadcast_shape(interp_term.shape[:-2], natural_vec.shape[:-1])
+        batch_shape = torch.broadcast_shapes(interp_term.shape[:-2], natural_vec.shape[:-1])
         expanded_interp_term = interp_term.expand(*batch_shape, *interp_term.shape[-2:])
         expanded_natural_vec = natural_vec.expand(*batch_shape, natural_vec.size(-1))
 
@@ -173,7 +173,7 @@ class CiqVariationalStrategy(_VariationalStrategy):
             device=self._variational_distribution.device,
         )
         ones = torch.ones_like(zeros)
-        res = MultivariateNormal(zeros, DiagLazyTensor(ones))
+        res = MultivariateNormal(zeros, DiagLinearOperator(ones))
         return res
 
     @property
@@ -203,14 +203,14 @@ class CiqVariationalStrategy(_VariationalStrategy):
         num_induc = inducing_points.size(-2)
         test_mean = full_output.mean[..., num_induc:]
         induc_induc_covar = full_covar[..., :num_induc, :num_induc].evaluate_kernel().add_jitter(1e-2)
-        induc_data_covar = full_covar[..., :num_induc, num_induc:].evaluate()
+        induc_data_covar = full_covar[..., :num_induc, num_induc:].to_dense()
         data_data_covar = full_covar[..., num_induc:, num_induc:].add_jitter(1e-4)
 
         # Compute interpolation terms
         # K_XZ K_ZZ^{-1} \mu_z
         # K_XZ K_ZZ^{-1/2} \mu_Z
         with settings.max_preconditioner_size(0):  # Turn off preconditioning for CIQ
-            interp_term = lazify(induc_induc_covar).sqrt_inv_matmul(induc_data_covar)
+            interp_term = to_linear_operator(induc_induc_covar).sqrt_inv_matmul(induc_data_covar)
 
         # Compute interpolated mean and variance terms
         # We have separate computation rules for NGD versus standard GD
@@ -222,9 +222,9 @@ class CiqVariationalStrategy(_VariationalStrategy):
             )
 
             # Compute the covariance of q(f)
-            predictive_var = data_data_covar.diag() - interp_term.pow(2).sum(dim=-2) + interp_var
+            predictive_var = data_data_covar.diagonal(dim1=-1, dim2=-2) - interp_term.pow(2).sum(dim=-2) + interp_var
             predictive_var = torch.clamp_min(predictive_var, settings.min_variance.value(predictive_var.dtype))
-            predictive_covar = DiagLazyTensor(predictive_var)
+            predictive_covar = DiagLinearOperator(predictive_var)
 
             # Also compute and cache the KL divergence
             if not hasattr(self, "_memoize_cache"):
@@ -240,10 +240,10 @@ class CiqVariationalStrategy(_VariationalStrategy):
             # Compute the covariance of q(f)
             middle_term = self.prior_distribution.lazy_covariance_matrix.mul(-1)
             if variational_inducing_covar is not None:
-                middle_term = SumLazyTensor(variational_inducing_covar, middle_term)
-            predictive_covar = SumLazyTensor(
+                middle_term = SumLinearOperator(variational_inducing_covar, middle_term)
+            predictive_covar = SumLinearOperator(
                 data_data_covar.add_jitter(1e-4),
-                MatmulLazyTensor(interp_term.transpose(-1, -2), middle_term @ interp_term),
+                MatmulLinearOperator(interp_term.transpose(-1, -2), middle_term @ interp_term),
             )
 
         # Compute the mean of q(f)
