@@ -1,13 +1,12 @@
 #!/usr/bin/env python3
 
 import torch
+from linear_operator.operators import KroneckerProductLinearOperator, RootLinearOperator
+from linear_operator.utils.interpolation import left_interp
 
 from .. import settings
 from ..distributions import MultitaskMultivariateNormal, MultivariateNormal
-from ..lazy import KroneckerProductLazyTensor, RootLazyTensor
 from ..module import Module
-from ..utils.broadcasting import _mul_broadcast_shape
-from ..utils.interpolation import left_interp
 from ._variational_strategy import _VariationalStrategy
 
 
@@ -19,7 +18,7 @@ def _select_lmc_coefficients(lmc_coefficients: torch.Tensor, indices: torch.Long
     lmc_coefficients: torch.Tensor ... x num_latents x ... x num_tasks
     indices: torch.Tesnor ... x N
     """
-    batch_shape = _mul_broadcast_shape(lmc_coefficients.shape[:-1], indices.shape[:-1])
+    batch_shape = torch.broadcast_shapes(lmc_coefficients.shape[:-1], indices.shape[:-1])
 
     # We will use the left_interp helper to do the indexing
     lmc_coefficients = lmc_coefficients.expand(*batch_shape, lmc_coefficients.shape[-1])[..., None]
@@ -204,8 +203,8 @@ class LMCVariationalStrategy(_VariationalStrategy):
 
             # Covar: ... x (N x num_tasks) x (N x num_tasks)
             latent_covar = latent_dist.lazy_covariance_matrix
-            lmc_factor = RootLazyTensor(lmc_coefficients.unsqueeze(-1))
-            covar = KroneckerProductLazyTensor(latent_covar, lmc_factor).sum(latent_dim)
+            lmc_factor = RootLinearOperator(lmc_coefficients.unsqueeze(-1))
+            covar = KroneckerProductLinearOperator(latent_covar, lmc_factor).sum(latent_dim)
             # Add a bit of jitter to make the covar PD
             covar = covar.add_jitter(settings.cholesky_jitter.value(dtype=mean.dtype))
 
@@ -222,7 +221,7 @@ class LMCVariationalStrategy(_VariationalStrategy):
 
             # Covar: ... x N x N
             latent_covar = latent_dist.lazy_covariance_matrix
-            lmc_factor = RootLazyTensor(lmc_coefficients.unsqueeze(-1))
+            lmc_factor = RootLinearOperator(lmc_coefficients.unsqueeze(-1))
             covar = (latent_covar * lmc_factor).sum(latent_dim)
             # Add a bit of jitter to make the covar PD
             covar = covar.add_jitter(settings.cholesky_jitter.value(dtype=mean.dtype))
