@@ -3,11 +3,13 @@
 import os
 import random
 import unittest
+from unittest.mock import MagicMock, patch
 import warnings
 from math import exp, pi
 
 import gpytorch
 import torch
+import linear_operator
 from gpytorch.distributions import MultivariateNormal
 from gpytorch.kernels import InducingPointKernel, RBFKernel, ScaleKernel
 from gpytorch.likelihoods import GaussianLikelihood
@@ -40,7 +42,7 @@ def make_data(cuda=False):
 class GPRegressionModel(gpytorch.models.ExactGP):
     def __init__(self, train_x, train_y, likelihood):
         super(GPRegressionModel, self).__init__(train_x, train_y, likelihood)
-        self.mean_module = ConstantMean(prior=SmoothedBoxPrior(-1e-5, 1e-5))
+        self.mean_module = ConstantMean(constant_prior=SmoothedBoxPrior(-1e-5, 1e-5))
         self.base_covar_module = ScaleKernel(RBFKernel(lengthscale_prior=SmoothedBoxPrior(exp(-5), exp(6), sigma=0.1)))
         self.covar_module = InducingPointKernel(
             self.base_covar_module, inducing_points=torch.linspace(0, 1, 32), likelihood=likelihood
@@ -65,41 +67,56 @@ class TestSGPRRegression(unittest.TestCase, BaseTestCase):
         if hasattr(self, "rng_state"):
             torch.set_rng_state(self.rng_state)
 
-    def test_sgpr_mean_abs_error(self):
+    def test_sgpr_mean_abs_error(self, cuda=False):
         # Suppress numerical warnings
         warnings.simplefilter("ignore", NumericalWarning)
 
-        train_x, train_y, test_x, test_y = make_data()
+        train_x, train_y, test_x, test_y = make_data(cuda=cuda)
         likelihood = GaussianLikelihood()
         gp_model = GPRegressionModel(train_x, train_y, likelihood)
         mll = gpytorch.mlls.ExactMarginalLogLikelihood(likelihood, gp_model)
 
-        # Optimize the model
-        gp_model.train()
-        likelihood.train()
+        if cuda:
+            gp_model = gp_model.cuda()
+            likelihood = likelihood.cuda()
 
-        optimizer = optim.Adam(gp_model.parameters(), lr=0.1)
-        for _ in range(30):
-            optimizer.zero_grad()
-            output = gp_model(train_x)
-            loss = -mll(output, train_y)
-            loss.backward()
-            optimizer.step()
+        # Mock cholesky
+        _wrapped_cholesky = MagicMock(
+            wraps=torch.linalg.cholesky_ex
+        )
+        with patch("torch.linalg.cholesky_ex", new=_wrapped_cholesky) as cholesky_mock:
 
-            # Check that we have the right LazyTensor type
-            kernel = likelihood(gp_model(train_x)).lazy_covariance_matrix.evaluate_kernel()
-            self.assertIsInstance(kernel, gpytorch.lazy.LowRankRootAddedDiagLazyTensor)
+            # Optimize the model
+            gp_model.train()
+            likelihood.train()
 
-        for param in gp_model.parameters():
-            self.assertTrue(param.grad is not None)
-            self.assertGreater(param.grad.norm().item(), 0)
+            optimizer = optim.Adam(gp_model.parameters(), lr=0.1)
+            for _ in range(30):
+                optimizer.zero_grad()
+                output = gp_model(train_x)
+                loss = -mll(output, train_y)
+                loss.backward()
+                optimizer.step()
 
-        # Test the model
-        gp_model.eval()
-        likelihood.eval()
+                # Check that we have the right LinearOperator type
+                kernel = likelihood(gp_model(train_x)).lazy_covariance_matrix.evaluate_kernel()
+                self.assertIsInstance(kernel, linear_operator.operators.LowRankRootAddedDiagLinearOperator)
 
-        test_preds = likelihood(gp_model(test_x)).mean
-        mean_abs_error = torch.mean(torch.abs(test_y - test_preds))
+            for param in gp_model.parameters():
+                self.assertTrue(param.grad is not None)
+                self.assertGreater(param.grad.norm().item(), 0)
+
+            # Test the model
+            gp_model.eval()
+            likelihood.eval()
+
+            test_preds = likelihood(gp_model(test_x)).mean
+            mean_abs_error = torch.mean(torch.abs(test_y - test_preds))
+            cholesky_mock.assert_called()  # We SHOULD call Cholesky...
+            for chol_arg in cholesky_mock.call_args_list:
+                first_arg = chol_arg[0][0]
+                self.assertTrue(torch.is_tensor(first_arg))
+                self.assertTrue(first_arg.size(-1) == gp_model.covar_module.inducing_points.size(-2))
 
         self.assertLess(mean_abs_error.squeeze().item(), 0.1)
 
@@ -109,7 +126,7 @@ class TestSGPRRegression(unittest.TestCase, BaseTestCase):
         self.assertGreater(test_vars.min().item() + 0.1, likelihood.noise.item())
         self.assertLess(
             test_vars.max().item() - 0.05,
-            likelihood.noise.item() + gp_model.covar_module.base_kernel.outputscale.item()
+            likelihood.noise.item() + gp_model.covar_module.base_kernel.outputscale.item(),
         )
 
         # Test on training data
@@ -123,62 +140,9 @@ class TestSGPRRegression(unittest.TestCase, BaseTestCase):
 
         if not torch.cuda.is_available():
             return
+
         with least_used_cuda_device():
-            train_x, train_y, test_x, test_y = make_data(cuda=True)
-            likelihood = GaussianLikelihood().cuda()
-            gp_model = GPRegressionModel(train_x, train_y, likelihood).cuda()
-            mll = gpytorch.mlls.ExactMarginalLogLikelihood(likelihood, gp_model)
-
-            # Test the model before optimization
-            gp_model.eval()
-            likelihood.eval()
-            test_preds = likelihood(gp_model(test_x)).mean
-            mean_abs_error = torch.mean(torch.abs(test_y - test_preds))
-            self.assertLess(mean_abs_error.squeeze().item(), 0.02)
-
-            # Test variances before optimization
-            test_vars = likelihood(gp_model(test_x)).variance
-            self.assertAllClose(test_vars, likelihood(gp_model(test_x)).covariance_matrix.diagonal(dim1=-1, dim2=-2))
-            self.assertGreater(test_vars.min().item() + 0.1, likelihood.noise.item())
-            self.assertLess(
-                test_vars.max().item() - 0.05,
-                likelihood.noise.item() + gp_model.covar_module.base_kernel.outputscale.item()
-            )
-
-            # Optimize the model
-            gp_model.train()
-            likelihood.train()
-
-            optimizer = optim.Adam(gp_model.parameters(), lr=0.1)
-            optimizer.n_iter = 0
-            for _ in range(25):
-                optimizer.zero_grad()
-                output = gp_model(train_x)
-                loss = -mll(output, train_y)
-                loss.backward()
-                optimizer.n_iter += 1
-                optimizer.step()
-
-            for param in gp_model.parameters():
-                self.assertTrue(param.grad is not None)
-                self.assertGreater(param.grad.norm().item(), 0)
-
-            # Test the model
-            gp_model.eval()
-            likelihood.eval()
-            test_preds = likelihood(gp_model(test_x)).mean
-            mean_abs_error = torch.mean(torch.abs(test_y - test_preds))
-
-            self.assertLess(mean_abs_error.squeeze().item(), 0.02)
-
-            # Test variances
-            test_vars = likelihood(gp_model(test_x)).variance
-            self.assertAllClose(test_vars, likelihood(gp_model(test_x)).covariance_matrix.diagonal(dim1=-1, dim2=-2))
-            self.assertGreater(test_vars.min().item() + 0.1, likelihood.noise.item())
-            self.assertLess(
-                test_vars.max().item() - 0.05,
-                likelihood.noise.item() + gp_model.covar_module.base_kernel.outputscale.item()
-            )
+            self.test_sgpr_mean_abs_error(cuda=True)
 
 
 if __name__ == "__main__":
