@@ -19,8 +19,8 @@ class TestRBFCovariance(unittest.TestCase):
         x2 = torch.randn(*batch_size, 6, 9)
         # Doesn't support ARD
         lengthscale = torch.randn(*batch_size).view(*batch_size, 1, 1) ** 2
-        res = gpytorch.functions.RBFCovariance().apply(x1, x2, lengthscale, sq_dist_func)
-        actual = sq_dist_func(x1, x2).div(-2 * lengthscale ** 2).exp()
+        res = gpytorch.functions.RBFCovariance.apply(x1, x2, lengthscale, sq_dist_func)
+        actual = sq_dist_func(x1, x2).div(-2 * lengthscale**2).exp()
         self.assertTrue(torch.allclose(res, actual))
 
     def test_backward(self):
@@ -28,7 +28,7 @@ class TestRBFCovariance(unittest.TestCase):
         x1 = torch.randn(*batch_size, 7, 9, dtype=torch.float64)
         x2 = torch.randn(*batch_size, 6, 9, dtype=torch.float64)
         lengthscale = torch.randn(*batch_size, dtype=torch.float64, requires_grad=True).view(*batch_size, 1, 1) ** 2
-        f = lambda x1, x2, l: gpytorch.functions.RBFCovariance().apply(x1, x2, l, sq_dist_func)
+        f = lambda x1, x2, l: gpytorch.functions.RBFCovariance.apply(x1, x2, l, sq_dist_func)
         try:
             torch.autograd.gradcheck(f, (x1, x2, lengthscale))
         except RuntimeError:
