@@ -1,13 +1,21 @@
 #!/usr/bin/env python3
 
+import pickle
 import unittest
 
 import torch
+from linear_operator.operators import DiagLinearOperator
 
+from gpytorch import settings
 from gpytorch.distributions import MultivariateNormal
-from gpytorch.lazy import DiagLazyTensor
-from gpytorch.likelihoods import DirichletClassificationLikelihood, FixedNoiseGaussianLikelihood, GaussianLikelihood
+from gpytorch.likelihoods import (
+    DirichletClassificationLikelihood,
+    FixedNoiseGaussianLikelihood,
+    GaussianLikelihood,
+    GaussianLikelihoodWithMissingObs,
+)
 from gpytorch.likelihoods.noise_models import FixedGaussianNoise
+from gpytorch.priors import GammaPrior
 from gpytorch.test.base_likelihood_test_case import BaseLikelihoodTestCase
 
 
@@ -17,6 +25,10 @@ class TestGaussianLikelihood(BaseLikelihoodTestCase, unittest.TestCase):
     def create_likelihood(self):
         return GaussianLikelihood()
 
+    def test_pickle_with_prior(self):
+        likelihood = GaussianLikelihood(noise_prior=GammaPrior(1, 1))
+        pickle.loads(pickle.dumps(likelihood))  # Should be able to pickle and unpickle with a prior
+
 
 class TestGaussianLikelihoodBatch(TestGaussianLikelihood):
     seed = 0
@@ -59,13 +71,13 @@ class TestFixedNoiseGaussianLikelihood(BaseLikelihoodTestCase, unittest.TestCase
             self.assertTrue(torch.equal(lkhd.noise, new_noise))
             # test __call__
             mean = torch.zeros(4, device=device, dtype=dtype)
-            covar = DiagLazyTensor(torch.ones(4, device=device, dtype=dtype))
+            covar = DiagLinearOperator(torch.ones(4, device=device, dtype=dtype))
             mvn = MultivariateNormal(mean, covar)
             out = lkhd(mvn)
             self.assertTrue(torch.allclose(out.variance, 1 + new_noise))
             # things should break if dimensions mismatch
             mean = torch.zeros(5, device=device, dtype=dtype)
-            covar = DiagLazyTensor(torch.ones(5, device=device, dtype=dtype))
+            covar = DiagLinearOperator(torch.ones(5, device=device, dtype=dtype))
             mvn = MultivariateNormal(mean, covar)
             with self.assertWarns(UserWarning):
                 lkhd(mvn)
@@ -73,6 +85,13 @@ class TestFixedNoiseGaussianLikelihood(BaseLikelihoodTestCase, unittest.TestCase
             obs_noise = 0.1 + torch.rand(5, device=device, dtype=dtype)
             out = lkhd(mvn, noise=obs_noise)
             self.assertTrue(torch.allclose(out.variance, 1 + obs_noise))
+            # test noise smaller than min_fixed_noise
+            expected_min_noise = settings.min_fixed_noise.value(dtype)
+            noise[:2] = 0
+            lkhd = FixedNoiseGaussianLikelihood(noise=noise)
+            expected_noise = noise.clone()
+            expected_noise[:2] = expected_min_noise
+            self.assertTrue(torch.allclose(lkhd.noise, expected_noise))
 
 
 class TestFixedNoiseGaussianLikelihoodBatch(BaseLikelihoodTestCase, unittest.TestCase):
@@ -127,13 +146,13 @@ class TestDirichletClassificationLikelihood(BaseLikelihoodTestCase, unittest.Tes
             self.assertTrue(torch.equal(lkhd.noise, new_noise))
             # test __call__
             mean = torch.zeros(6, device=device, dtype=dtype)
-            covar = DiagLazyTensor(torch.ones(6, device=device, dtype=dtype))
+            covar = DiagLinearOperator(torch.ones(6, device=device, dtype=dtype))
             mvn = MultivariateNormal(mean, covar)
             out = lkhd(mvn)
             self.assertTrue(torch.allclose(out.variance, 1 + new_noise))
             # things should break if dimensions mismatch
             mean = torch.zeros(5, device=device, dtype=dtype)
-            covar = DiagLazyTensor(torch.ones(5, device=device, dtype=dtype))
+            covar = DiagLinearOperator(torch.ones(5, device=device, dtype=dtype))
             mvn = MultivariateNormal(mean, covar)
             with self.assertWarns(UserWarning):
                 lkhd(mvn)
@@ -145,5 +164,61 @@ class TestDirichletClassificationLikelihood(BaseLikelihoodTestCase, unittest.Tes
             self.assertTrue(torch.allclose(out.variance, 1.0 + obs_targets))
 
 
+class TestGaussianLikelihoodwithMissingObs(BaseLikelihoodTestCase, unittest.TestCase):
+    seed = 42
+
+    def create_likelihood(self):
+        return GaussianLikelihoodWithMissingObs()
+
+    def test_missing_value_inference(self):
+        """
+        samples = mvn samples + noise samples
+        In this test, we try to recover noise parameters when some elements in
+        'samples' are missing at random.
+        """
+
+        torch.manual_seed(self.seed)
+
+        mu = torch.zeros(2, 3)
+        sigma = torch.tensor([[[1, 0.999, -0.999], [0.999, 1, -0.999], [-0.999, -0.999, 1]]] * 2).float()
+        mvn = MultivariateNormal(mu, sigma)
+        samples = mvn.sample(torch.Size([10000]))  # mvn samples
+
+        noise_sd = 0.5
+        noise_dist = torch.distributions.Normal(0, noise_sd)
+        samples += noise_dist.sample(samples.shape)  # noise
+
+        missing_prop = 0.33
+        missing_idx = torch.distributions.Binomial(1, missing_prop).sample(samples.shape).bool()
+        samples[missing_idx] = float("nan")
+
+        likelihood = GaussianLikelihoodWithMissingObs()
+
+        # check that the missing value fill doesn't impact the likelihood
+
+        likelihood.MISSING_VALUE_FILL = 999.0
+        like_init_plus = likelihood.log_marginal(samples, mvn).sum().data
+
+        likelihood.MISSING_VALUE_FILL = -999.0
+        like_init_minus = likelihood.log_marginal(samples, mvn).sum().data
+
+        torch.testing.assert_allclose(like_init_plus, like_init_minus)
+
+        # check that the correct noise sd is recovered
+
+        opt = torch.optim.Adam(likelihood.parameters(), lr=0.05)
+
+        for _ in range(100):
+            opt.zero_grad()
+            loss = -likelihood.log_marginal(samples, mvn).sum()
+            loss.backward()
+            opt.step()
+
+        assert abs(float(likelihood.noise.sqrt()) - 0.5) < 0.02
+
+        # Check log marginal works
+        likelihood.log_marginal(samples[0], mvn)
+
+
 if __name__ == "__main__":
     unittest.main()
