@@ -20,7 +20,7 @@ class TestGridVariationalGP(VariationalTestCase, unittest.TestCase):
     ):
         class _SVGPRegressionModel(gpytorch.models.ApproximateGP):
             def __init__(self):
-                variational_distribution = distribution_cls(num_inducing ** 2, batch_shape=batch_shape)
+                variational_distribution = distribution_cls(num_inducing**2, batch_shape=batch_shape)
                 variational_strategy = strategy_cls(self, num_inducing, [(-3, 3), (-3, 3)], variational_distribution)
                 super().__init__(variational_strategy)
                 if constant_mean:
@@ -75,6 +75,10 @@ class TestGridVariationalGP(VariationalTestCase, unittest.TestCase):
         self.assertFalse(cholesky_mock.called)
         self.assertFalse(ciq_mock.called)
 
+    def test_fantasy_call(self, *args, **kwargs):
+        with self.assertRaises(AttributeError):
+            super().test_fantasy_call(*args, **kwargs)
+
 
 class TestGridPredictiveGP(TestGridVariationalGP):
     @property
