@@ -40,6 +40,14 @@ class TestUnwhitenedVariationalGP(VariationalTestCase, unittest.TestCase):
         self.assertFalse(ciq_mock.called)
         self.assertEqual(cholesky_mock.call_count, 1)  # One to compute cache, that's it!
 
+    def test_fantasy_call(self, *args, **kwargs):
+        # we only want to check CholeskyVariationalDistribution
+        if self.distribution_cls is gpytorch.variational.CholeskyVariationalDistribution:
+            return super().test_fantasy_call(*args, **kwargs)
+
+        with self.assertRaises(AttributeError):
+            super().test_fantasy_call(*args, **kwargs)
+
 
 class TestUnwhitenedPredictiveGP(TestUnwhitenedVariationalGP):
     @property
